// Copyright (c) 2017  John Abbott
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"


//----------------------------------------------------------------------
const string ShortDescription =
  "This example shows how to use a CpuTimeLimit object to limit        \n"
  "the CPU time used in a function.  Compare this example with that in \n"
  "ex-CpuTimeLimit2.C.";

const string LongDescription =
  "This example shows how to use a CpuTimeLimit object to limit        \n"
  "the CPU time used in a function.  Compare this example with that in \n"
  "ex-CpuTimeLimit2.C.";

//----------------------------------------------------------------------

namespace CoCoA
{

  // New function to compute a reduced GBasis with a limit on CPU time.
  // This will throw a TimeoutException if the computation takes too long.
  // The built-in fn GBasisTimeout is almost identical to this.
  vector<RingElem> RedGBasisTimeout(const ideal& I, double Tmax/*seconds*/)
  {
    CpuTimeLimit timeout(Tmax);
    return ReducedGBasis(I); // <-- NB ReducedGBasis calls CheckForInterrupt
  }

  void program()
  {
    GlobalManager CoCoAFoundations;

    cout << ShortDescription << endl;

    ring P = NewPolyRing(RingQQ(), symbols("x,y,z"), lex);
    const RingElem f1 = RingElem(P, "x^3 -x*y*z +z");
    const RingElem f2 = RingElem(P, "x^2*y +x +z^2");
    const RingElem f3 = RingElem(P, "x*y^2 +x +y^3");
    ideal I = ideal(f1,f2,f3);
    
    // Now call fn defined above: if TimeoutException is thrown, it
    // will be caught (and a message printed).
    try
    {
      const vector<RingElem> RGB = RedGBasisTimeout(I, 3.0); // max CPU is 3.0s
      cout << "len(RGB) = " << len(RGB) << endl;
    }
    catch (const CoCoA::TimeoutException&)
    {
      // Just print a message saying it timed out.
      cout << "-------------------------------" << endl
           << ">>>  Computation timed out  <<<" << endl
           << "-------------------------------" << endl;
    }
  }

} // end of namespace CoCoA

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    CoCoA::program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  CoCoA::BuildInfo::PrintAll(cerr);
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-CpuTimeLimit1.C,v 1.8 2017/10/16 15:25:32 abbott Exp $
// $Log: ex-CpuTimeLimit1.C,v $
// Revision 1.8  2017/10/16 15:25:32  abbott
// Summary: Time-out now announced on cout (instead of clog)
//
// Revision 1.7  2017/09/13 08:45:33  abbott
// Summary: Now exit status is 0 even wen time-out occurs.
//
// Revision 1.6  2017/09/06 13:55:24  abbott
// Summary: Updated comment: exception is now called TimeoutException
//
// Revision 1.5  2017/08/08 13:46:54  abbott
// Summary: Added comments
//
// Revision 1.4  2017/07/23 15:27:24  abbott
// Summary: Renamed demo fn to avoid name clash
//
// Revision 1.3  2017/07/22 12:55:15  abbott
// Summary: Used PrintInFrame
//
// Revision 1.2  2017/07/21 13:17:57  abbott
// Summary: Minor improvements
//
// Revision 1.1  2017/07/15 15:15:44  abbott
// Summary: Added examples for CpuTimeLimit
//
//
