//   Copyright (c)  2017  John Abbott,  Anna M. Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/CpuTimeLimit.H"
#include "CoCoA/error.H"
#include "CoCoA/time.H"

#include <iostream>
using namespace std;

namespace CoCoA
{

  namespace // anonymous
  {

    /*global*/ /*NOT ATOMIC*/ volatile double CPU_TIME_LIMIT = 0.0; // 0.0 means "no time limit"

  } // end of anonymous namespace
  
  void CheckForTimeout(const char* const context)
  {
    if (CPU_TIME_LIMIT > 0.0 && CPU_TIME_LIMIT < CpuTime())
      throw TimeoutException(context);
  }

  // void CheckForTimeout(const std::string& context)
  // {
  //   if (CPU_TIME_LIMIT > 0.0 && CPU_TIME_LIMIT < CpuTime())
  //     throw TimeoutException(context);
  // }


  CpuTimeLimit::CpuTimeLimit(double t)
  {
    if (t < 0) CoCoA_ERROR(ERR::NotPositive, "CpuTimeLimit ctor");

    if (t > 1000000.0) // arbitrary upper limit
      CoCoA_ERROR(ERR::ArgTooBig, "CpuTimeLimit ctor");

    myStartTime = CpuTime();
    myPrevExpirationTime = CPU_TIME_LIMIT;
    myExpirationTime = t + myStartTime;
    if (CPU_TIME_LIMIT > 0 && myExpirationTime > CPU_TIME_LIMIT)
      myExpirationTime = CPU_TIME_LIMIT;
    CPU_TIME_LIMIT = myExpirationTime;
  }


  std::ostream& CpuTimeLimit::myOutputSelf(std::ostream& out) const
  {
    if (myPrevExpirationTime < 0)
    {
      return out << "CpuTimeLimit(DEACTIVATED)";
    }
    return out << "CpuTimeLimit(ExpirationTime=" << myExpirationTime << ")";
  }

  
  std::ostream& operator<<(std::ostream& out, const CpuTimeLimit& CpuLimit)
  {
    CpuLimit.myOutputSelf(out);
    return out;
  }


  void CpuTimeLimit::myDeactivate()
  {
    if (myPrevExpirationTime < 0) return; // already deactivated
    CPU_TIME_LIMIT = myPrevExpirationTime;
    myPrevExpirationTime = -1.0; // negative ==> deactivated
  }


  CpuTimeLimit::~CpuTimeLimit()
  {
    myDeactivate();
  }


  //-------------------------------------------------------
  // Must define this because of virtual mem fns.
  TimeoutException::~TimeoutException()
  {}


  std::ostream& TimeoutException::myOutputSelf(std::ostream& out) const
  {
    if (!out) return out;  // short-cut for bad ostreams
    out << "CoCoA::TimeoutException(context=\"" << myContext << "\")";
    return out;
  }
    
} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/CpuTimeLimit.C,v 1.7 2017/09/06 14:08:50 abbott Exp $
// $Log: CpuTimeLimit.C,v $
// Revision 1.7  2017/09/06 14:08:50  abbott
// Summary: Changed name to TimeoutException
//
// Revision 1.6  2017/07/23 15:32:32  abbott
// Summary: Fixed STUPID bug in myDeactivate
//
// Revision 1.5  2017/07/22 13:03:02  abbott
// Summary: Added new exception InterruptdByTimeout; changed rtn type of myOutputSelf
//
// Revision 1.4  2017/07/21 15:06:10  abbott
// Summary: Major revision -- no longer needs BOOST
//
// Revision 1.3  2017/07/21 13:21:22  abbott
// Summary: Split olf interrupt into two ==> new file SignalWatcher; refactored interrupt and CpuTimeLimit
//
// Revision 1.2  2017/07/15 15:44:44  abbott
// Summary: Corrected error ID name (to ArgTooBig)
//
// Revision 1.1  2017/07/15 15:17:48  abbott
// Summary: Added CpuTimeLimit
//
//
